#include <cstring>
#include "../common/util_classes.h"
#include "RusCorpHtml.h"
#include "MyStem2Dialing.h"
#include "../common/MorphXmlToken.h"




//      q   Items
//        ,     "<"  
//  ">",    <>   
bool ReadItems (const string& q, vector<CHtmlItem>& Items, const vector<size_t>& line_breaks)
{
	Items.clear();
	const string comment_start = "<!--";
	const string comment_end = "-->";
	const string comment_end2 = "--!>";
	int LineNo = 0;
	for (int i=0; i < q.length(); i++)
	{	
		if (i >= line_breaks[LineNo])
			LineNo++;

		// pass comments 
		if (q.substr(i,  comment_start.length()) == comment_start)
		{
			for (; i < q.length(); i++)
			{
				if (q.substr(i,  comment_end.length()) == comment_end)
				{
					i+=(int)comment_end.length();
					break;
				}
				if (q.substr(i,  comment_end2.length()) == comment_end2)
				{
					i+=(int)comment_end2.length();
					break;
				}
			}
		}

		if (q[i] == '<')
		{
			CHtmlItem Item;
			Item.m_Offset = i;
			Item.m_LineNo = LineNo;
			for (; i < q.length(); i++)
				if (q[i] == '>')
					break;
			if (i == q.length()) 
			{
				fprintf (stderr, "cannot parse starting from %s", q.substr(i, min((size_t)Item.m_Offset+20, q.length())).c_str());
				return false;
			};
			Item.m_Item = q.substr(Item.m_Offset, i-Item.m_Offset+1);
			Items.push_back(Item);
		}
		else
			if (isspace((BYTE)q[i]) )
				continue;
			else
			if (q.length()> 5 && q.substr(i, 6) == "&nbsp;")
			{
				i+=5;
				continue;
			}
			else
			{
				// normal text 
				CHtmlItem Item;
				Item.m_Offset = i;
				Item.m_LineNo = LineNo;
				for(; i < q.length()&& ((!isspace((BYTE)q[i]) && (q[i] != '<'))); i++);
				if (i == q.length()) 
				{
					Item.m_Item = q.substr(Item.m_Offset, i-Item.m_Offset);
					Items.push_back(Item);
					return true;
				};
				Item.m_Item = q.substr(Item.m_Offset, i-Item.m_Offset);
				Items.push_back(Item);
				i--;
			};
	};
	return true; 
};

bool ParseGrammems (StringTokenizer& grammems_tok, string& pos, set<string>& grammems, bool bSupressMorphErrors)
{
	while (grammems_tok())
	{
		string mystem_gram = grammems_tok.val();
		string dialing_grammem = Grammem_MyStem2Dialing(mystem_gram);
		if (dialing_grammem.empty())
		{
			if (!bSupressMorphErrors)
				fprintf(stderr, "unknown grammem \"%s\"\n", mystem_gram.c_str());
			continue;
		};

		if (dialing_grammem == "")
			pos = "";
		else
		if (dialing_grammem == "")
			pos = "";
		else
		if (dialing_grammem == "")
			pos = "";

		if (mystem_gram== "Ash")
		{
			pos = "";
			grammems.insert("");
		}
		else
			grammems.insert (dialing_grammem);
	}
	return  true;
}



//=======================================

string CWordInfo::GetXml() const 
{
	string Res = Format("<%s", m_Type.c_str());
	Res += ">";
	size_t stress = m_Token.find("`");
	string Token = m_Token;
	if (stress == string::npos)
		stress = 255;
	else
	{
		Token.erase(stress,1);
	}
	Trim(Token);
	if (Token.empty())
		return "";
	
	
	if (m_Type == XmlWordType)
		Res += " " + Token + "\n";
	else
		Res += ConvertASCIIToHtmlSymbols(Token);


	for (size_t i=0; i < m_Interps.size(); i++)
	{	
		const CMorphInterpretation& I  = m_Interps[i];
		Res += Format("<ana lemma=\"%s\" pos=\"%s,\" gram=\"%s\"  stress=\"%i\"/>\n", 
			I.m_Lex.c_str(), I.m_Pos.c_str(), I.m_Gr.c_str(), stress);	
	}

	Res += Format("</%s>\n", m_Type.c_str());
	return Res;
}


//==============================================
bool ConvertAnnot (string s, CWordInfo& Res, bool bSupressMorphErrors)
{
	//<w lex='' gr='A=sg,m,loc,plen'> ->	<ana lemma="" pos="" gram=",,,"> 
	for (size_t i=0; i < s.length(); i++)
	{
		if ((unsigned char)s[i] == 145)
			s[i] = '\'';

		if ((unsigned char)s[i] == 146)
			s[i] = '\'';
	}


	size_t field_start = s.find_first_not_of(" >", strlen("<ana"));
	string interps;
	CMorphInterpretation I;

	while (field_start != string::npos)
	{
		size_t field_end = s.find_first_of(" =", field_start);
		string field = s.substr(field_start, field_end-field_start);
		size_t eq = s.find("=", field_end);
		if (eq == string::npos)
		{
			fprintf (stderr, "bad format %s\n", s.c_str());
			return false;
		}
		size_t value_start=  s.find("'",eq);
		if (value_start == string::npos)
		{
			fprintf (stderr, "bad format %s\n", s.c_str());
			return false;
		}
		value_start++;
		size_t value_end=  s.find("'",value_start);
		if (value_end == string::npos)
		{
			fprintf (stderr, "bad format %s\n", s.c_str());
			return false;
		}
		//     
		if (value_end +1 != s.length())
			if	(		(s.substr(value_end+1) != "/>")
					&&	((unsigned char)s[value_end+1] != ' ')
				)
			{
				value_end =  s.find("'",value_end+1);
				if (value_end == string::npos)
				{
					fprintf (stderr, "bad format %s\n", s.c_str());
					return false;
				}
			}
		string value = s.substr(value_start, value_end-value_start);
		if (field  == "lex")
		{
			I.m_Lex = value;

		}
		else
		if (field  == "gr")
			interps = value;
		else
		if (field  == "mw")
		{
			// do nothing
			
		}
		else
		{
			fprintf (stderr, "unknown field \"%s\" in line \"%s\"\n",field.c_str(), s.c_str());
			return false;
		}
		field_start = s.find_first_not_of(" >", value_end+1);

		string rest = s.substr(field_start);
		if (rest  == "/>")
			break;
	}


	StringTokenizer diff_grams(interps.c_str(),"(|,) =");
	int count = 0;
	set<string> grammems;
	vector<string> poses;
	while (diff_grams())
	{
		count++;
		string mystem_gram = diff_grams.val();
		if (count == 1)
		{
			StringTokenizer aa(mystem_gram.c_str(),"/");
			while (aa())
			{
				string hh = POS_MyStem2Dialing(aa.val());
				if (!hh.empty())
					poses.push_back(hh );
			}
			continue;
		}
		
		string dialing_grammem = Grammem_MyStem2Dialing(mystem_gram);
		if (dialing_grammem.empty())
		{
			if (!bSupressMorphErrors)
				fprintf(stderr, "unknown grammem \"%s\"\n", mystem_gram.c_str());
			continue;
		};

		if (dialing_grammem == "")
			poses.resize(1, "");
		else
		if (dialing_grammem == "")
			poses.resize(1, "");
		else
		if (dialing_grammem == "")
			poses.resize(1, "");

		if (mystem_gram== "Ash")
		{
			poses.resize(1, "");
			grammems.insert("");
		}
		else
			grammems.insert (dialing_grammem);
	}
	
	for (set<string>::const_iterator it = grammems.begin();  it != grammems.end();  it++)
		I.m_Gr += *it + ",";

	for (size_t i=0; i < poses.size(); i++)
	{
		I.m_Pos = poses[i];
		Res.m_Interps.push_back(I);
	}

	return true;
};




string ReadFileToString(FILE* in_fp, vector<size_t>& line_breaks)
{
	string q;
	char buffer[10000];
	while (fgets(buffer, 10000, in_fp)) 
	{
		q += buffer;
		line_breaks.push_back(q.length());
	}
	return q;
}


int ReadHeader(const vector<CHtmlItem>& Items, string& result_bibl, string InFileName )
{
	result_bibl = "";
	bool bDate = false;
	bool bTitle = false;
	int i = 0;
	for (; i < Items.size(); i++)
	{
		string h = Items[i].m_Item;
		if (h == "<body>")
		{

				break;
		}

		if (h.substr(0, strlen("<meta ")) == "<meta " )
		{
			int j = (int)strlen("<meta ");
			j = (int)h.find_first_not_of(" \t", j);
			if ( (j != string::npos) && (h.substr(j, strlen("name")) == "name" ))
			{	
				j = (int)h.find_first_not_of(" ='\"\t", j+strlen("name"));
				
				if ( j != string::npos  )
				{
					size_t end_name = string::npos;
					if (h.substr(j, strlen("author")) == "author")
						end_name = strlen("author");

					if (h.substr(j, strlen("title")) == "title")
					{
						bTitle = true;
						end_name = strlen("title");
					}

					if (h.substr(j, strlen("date")) == "date")
					{
						bDate = true;
						end_name = strlen("date");
					}

				}
			};
			
		}

	}

	if (!bTitle)
		result_bibl += Format("<title>%s</title>\n", InFileName.c_str());
	if (!bDate)
		result_bibl += Format("<date>1900</date>\n");
	return i;
}

bool IsSequenceOfPuncts (const string& s )
{
	for (size_t i=0; i < s.length(); i++)
		if (!ispunct((unsigned char)s[i]))
			return false;

	return true;
}

int ReadSentenceFromItems(vector<CHtmlItem>& Items, int StartItemNo, vector<CWordInfo>& Result, bool bSupressMorphErrors)
{	
	Result.clear();
	int i;
	for (i = StartItemNo; i < Items.size(); i++)
	{
		string h = Items[i].m_Item;
		Items[i].m_InterpNo = -1;
		if (		(h == "<w>")
				||	(h == "<w >")
			)
		{
			Items[i].m_bIgnore = true;		
			i++;	
			if (i >= Items.size())
			{
				fprintf (stderr, "bad format at line %i\n", Items[i].m_LineNo+1);
				return -1;
			};
			if (Items[i].m_Item == "</w>")
			{
				fprintf (stderr, "empty word is found\n");						
				i++;
				continue;
				
			}
			CWordInfo Info;
			while (i < Items.size() && Items[i].m_Item.substr(0,5) == "<ana ")
			{
				if (!ConvertAnnot(Items[i].m_Item, Info,bSupressMorphErrors))
				{
					fprintf (stderr, "bad format in annotation %s \n", h.c_str());
					return -1;
				}
				Items[i].m_bIgnore = true;
				i++;
				
			}
			if (i == Items.size())
			{
				fprintf (stderr, "file is not finished\n");
				return -1;

			}
			if (!IsSequenceOfPuncts(Items[i].m_Item))
				Info.m_Type = XmlWordType;
			else
				Info.m_Type = XmlPunctType;

			Info.m_Token = Items[i].m_Item;

			if (		(Info.m_Type == XmlWordType)
					&&	!CheckLanguage(Info.m_Token,morphRussian)  //    
					&&	!CheckLanguage(Info.m_Token,morphEnglish) 
					&&	!CheckLanguage(Info.m_Token,morphDigits) 
					&&	Info.m_Token.length() < 255	
				)
			{
				char buffer[256];
				if (force_to_rus(buffer, Info.m_Token.c_str(), Info.m_Token.length()))
				{
					

					//fprintf (stderr, "coercing to russian ABC (token %s)\n", Info.m_Token.c_str());

					buffer[Info.m_Token.length()] = 0;
					Info.m_Token = buffer;
				
				}
			}

			if ( (Info.m_Token == "</w>") || (Info.m_Token == "<w>") )
			{
				fprintf (stderr, "Empty word at line %i\n", Items[i].m_LineNo+1);
				return -1;
			}
			
			Items[i].m_InterpNo = (int)Result.size();
			Result.push_back(Info);

			i++;
			if ((i == Items.size()) ||  (Items[i].m_Item != "</w>" ))
			{
				fprintf (stderr, "cannot find </w> at line %i (token %s)\n", Items[i].m_LineNo+1, Info.m_Token.c_str());
				return -1;
			}
			Items[i].m_bIgnore = true;
		}
		else
			if ( (h == "<se>") || (h == "<st>") || (h.substr(0, 4) == "<se "))
			{
				if (!Result.empty())
					break;
			}
		else
			if ( (h.substr(0,3) == "<cl") && (h.find("_") != string::npos))
			{
				size_t Count = 0;
				for (; i < Items.size(); i++)
				{
					if (Items[i].m_Item == "</cl>")
						break;
					
					if (Count++ > 1000) 
					{
						fprintf (stderr, "the foreign lexeme is too long\n");
						return -1;
					}
				}

				CWordInfo Info;
				Info.m_Type = XmlPunctType;
				Info.m_Token = "unk";
				Items[i].m_InterpNo = (int)Result.size();
				Result.push_back(Info);
			}
		else
			if (h[0] != '<')
			{
				if (CheckLanguage(h, morphEnglish) && !IsSequenceOfPuncts(h))
				{
					//  
					CWordInfo Info;
					Info.m_Type = XmlWordType;
					Info.m_Token = h;
					CMorphInterpretation I;
					I.m_Lex = h;
					RmlMakeLower(I.m_Lex, morphEnglish);
					Info.m_Interps.push_back(I);
					Items[i].m_InterpNo = (int)Result.size();
					Result.push_back(Info);

				}
				else
				for (int start = 0; start < h.length(); )
				{
					int end = start+1;
					for(; h[start] == h[end]; end++);
					string input_s = h.substr(start, end-start);
					//string  to_xml = ConvertASCIIToHtmlSymbols(input_s);
					CWordInfo Info;
					Info.m_Type = XmlPunctType;
					Info.m_Token = input_s;
					Items[i].m_InterpNo = (int)Result.size();
					Result.push_back(Info);
					start = end;
				};
			}

	};
	return i;
};

void PrintXmlSentence (FILE* out_fp, const vector<CWordInfo>& Result)
{
	fprintf (out_fp, "<s>\n");
	for (size_t i = 0; i < Result.size(); i++)
	{
		string s = Result[i].GetXml();
		if (!s.empty())
		fprintf (out_fp, "%s", s.c_str());
	}
	fprintf (out_fp, "</s>\n");
}

void PrintTextSentence (FILE* out_fp, const vector<CWordInfo>& Result)
{
	for (size_t i = 0; i < Result.size(); i++)
	{
		fprintf (out_fp, "%s", Result[i].m_Token.c_str());
		if (i+1 != Result.size())
			fprintf (out_fp, " ");
	}
	fprintf (out_fp, "\n");
}





//#pragma optimize( "", on )

int PrintXmlFile(FILE* out_fp, vector<CHtmlItem>& Items, string bibl, int StartItemNo)
{
	//fprintf (stderr, "===================== print xml ================\n");
	fprintf (out_fp, "<?xml version=\"1.0\" encoding=\"windows-1251\" ?>\n");
	fprintf (out_fp, "<xml>\n");
	fprintf (out_fp, "<bibl>\n%s</bibl>\n", bibl.c_str());
	fprintf (out_fp, "<text>\n");
	vector<CWordInfo> Result;
	int SentCount = 0;
	while	(		( (StartItemNo = ReadSentenceFromItems(Items, StartItemNo,Result,false)) != -1)
				&&	(StartItemNo < Items.size()) 
			)
	{
		
		PrintXmlSentence(out_fp, Result);
		SentCount++;
	};
	if (StartItemNo == -1)
		return -1;
	
	if (!Result.empty())
	{
		PrintXmlSentence(out_fp, Result);
		SentCount++;
	}

	fprintf (out_fp, "</text>\n");
	fprintf (out_fp, "</xml>\n");
	
	return  SentCount;
}

int PrintWordFormFile(FILE* out_fp, vector<CHtmlItem>& Items, int StartItemNo, const string& InFileContents)
{
	vector<CWordInfo> Result;
	int SentCount = 0;
	size_t SaveStartItemNo = StartItemNo;
	while	(		( (StartItemNo = ReadSentenceFromItems(Items, StartItemNo,Result, false)) != -1)
				&&	(StartItemNo < Items.size()) 
			)
	{
		for (size_t i=0; i < Result.size(); i++)
			if (Result[i].m_Type == XmlWordType)
				fprintf(out_fp, "%s\n", Result[i].m_Token.c_str());
		SaveStartItemNo = StartItemNo;
		SentCount++;

	}

	if (StartItemNo == -1)
		return -1;

	if (!Result.empty())
		SentCount++;

	for (size_t i=0; i < Result.size(); i++)
		if (Result[i].m_Type == XmlWordType)
			fprintf(out_fp, "%s\n", Result[i].m_Token.c_str());

	return SentCount;
}


bool GetHtmlItems(string InFileName, vector<CHtmlItem>& Items, string& InFileContents) 
{
	Items.clear();
	FILE* in_fp = fopen (InFileName.c_str(), "r");
	if (!in_fp) 
	{
		fprintf (stderr, "cannot open  %s\n", InFileName.c_str());
		return false;
	};

	vector<size_t> line_breaks;
	InFileContents = ReadFileToString(in_fp, line_breaks);
	fclose (in_fp);
	
	
	if (!ReadItems( InFileContents, Items, line_breaks))
		return false;
	return  true;
}
