#include <stdio.h>
#include "../common/SyntaxHolder.h"

struct  COperInfo {
	const char*	m_Situation;
	const char*	m_FuncVerb;
};
const size_t OperInfosCount = 1;
COperInfo OperInfos[OperInfosCount] = 
{
	{"DIENST", "LEISTEN"}
};

struct CCollocOccur 
{
	size_t  m_SituationUnitNo;
	size_t  m_FuncVerbUnitNo;
};


inline const CSynHomonym& GetHom (const CClause& Clause, const CMorphVariant& Variant, size_t UnitNo)
{
	const CSynUnit& U = Variant.m_SynUnits[UnitNo];
	if (U.m_Type == EWord)
		return Clause.GetWords()[U.m_SentPeriod.m_iFirstWord].m_Homonyms[U.m_iHomonymNum];
	else
	{
		int SubClauseNum = Clause.m_pSent->FindClauseIndexByPeriod(U.m_SentPeriod);
		assert (SubClauseNum != -1);
		const CClause& SubClause = Clause.m_pSent->m_Clauses[SubClauseNum];
		if (U.m_iClauseTypeNum == -1)
			return Clause.GetWords()[SubClause.m_iLastWord].m_Homonyms[0];

		const CWordAndHomonym& W = SubClause.m_vectorTypes[U.m_iClauseTypeNum].m_Root;
		return Clause.GetWords()[W.m_WordNo].m_Homonyms[W.m_HomonymNo];
	};
};

void WriteCurrentSentence(const CSentence& Sentence, FILE* fp)
{
	string S;
	for (int WordNo = 0; WordNo < Sentence.m_Words.size(); WordNo++)
	{
		S += Sentence.m_Words[WordNo].m_strWord.c_str();
		S += " ";
	};
	fprintf (fp, "%s\n", S.c_str());
};



// 1. Dienst : Akk, -not-subj-, main word of NP
// leisten : aktive clause root 
// 
//  examples:
//  Ich werde Dienste leisten;
//  Ich wrde Dienste leisten;
//  Ich habe Dienste geleistet;
// 
// but not 
// Dienst wurde geleistet 

bool CheckGrammar1(const CClause& Clause, const CMorphVariant& Variant,  const CCollocOccur& Occur)
{
	if (Variant.m_SynUnits[Occur.m_FuncVerbUnitNo].m_bReflexive) return false;

	const CSynUnit& SitUnit = Variant.m_SynUnits[Occur.m_SituationUnitNo];
	int SitWordNo = SitUnit.m_SentPeriod.m_iFirstWord;
	
	const CSynHomonym&  SitHom = GetHom(Clause, Variant, Occur.m_SituationUnitNo);
	const CSynHomonym&  VerbHom = GetHom(Clause, Variant, Occur.m_FuncVerbUnitNo);
	int VerbWordNo = Variant.m_SynUnits[Occur.m_FuncVerbUnitNo].m_SentPeriod.m_iFirstWord;

	int GroupNo = Variant.m_vectorGroups.get_maximal_group_no(SitWordNo);

	if	(GroupNo != -1)
	{
		const CGroup& G = Variant.m_vectorGroups.GetGroups()[GroupNo];
		if	(		(G.m_MainWordNo != SitWordNo)
				&&	(G.m_GroupType != Clause.GetOpt()->GetGroupTypebyName("NP_SIMIL"))
			)
		return false;
	};

	if ( !SitUnit.HasGrammem(gAkkusativ) )
		return false;

	if (Variant.m_Subjects.size() == 1)
		if	(Variant.GetFirstSubject() == Occur.m_SituationUnitNo)
			return false;

	// 
	//if	(Variant.m_iPredk != Occur.m_FuncVerbUnitNo)
	//	return false;
	//
	// because of "Ich muss den Dienst leisten"


	if	(			(( VerbHom.m_iGrammems & gAllVerbForms) == 0)
				&&	(Clause.m_pSent->FindFirstAuxVerb(VerbWordNo) == -1)
		)
		return false;

	// check active
	if (VerbHom.m_bPassive) return false;


	return true;
};


bool IsSubjectOrSimilar(const CClause& Clause, const CMorphVariant& Variant,  int UnitNo)
{
	for (size_t i=0; i <  Variant.m_Subjects.size(); i++)
	{
		if	(Variant.m_Subjects[i] == UnitNo) return true;
		int GroupNo = Variant.m_vectorGroups.get_maximal_group_no(Variant.m_Subjects[i]);
		if	(GroupNo != -1)
		{
			const CGroup& G = Variant.m_vectorGroups.GetGroups()[GroupNo];
			if (G.m_GroupType == Clause.GetOpt()->GetGroupTypebyName("NP_SIMIL"))
				return true;
		};
	};
	return false;
};


// 1. Dienst : subj, main word of NP
//  examples:
//  unter welchen Verhltnissen der Dienst geleistet wurde
//  Der Dienst wird geleistet 

bool CheckGrammar2(const CClause& Clause, const CMorphVariant& Variant,  const CCollocOccur& Occur)
{
	

	const CSynUnit& SitUnit = Variant.m_SynUnits[Occur.m_SituationUnitNo];
	const CSynHomonym&  SitHom = GetHom(Clause, Variant, Occur.m_SituationUnitNo);
	const CSynUnit& VerbUnit = Variant.m_SynUnits[Occur.m_FuncVerbUnitNo];
	const CSynHomonym&  VerbHom = GetHom(Clause, Variant, Occur.m_FuncVerbUnitNo);

	if (VerbUnit.m_bReflexive) return false;

	if (!IsSubjectOrSimilar(Clause, Variant,Occur.m_SituationUnitNo))
		return false;

	// check passive
	if (!VerbHom.m_bPassive) return false;
	 
	return true;
};

// in one NP
//  examples:
// Fr die geleisteten Dienste . 
// fr geleistete Arbeiten und Dienste
bool CheckGrammar3(const CClause& Clause, const CMorphVariant& Variant,  const CCollocOccur& Occur)
{
	if (Variant.m_SynUnits[Occur.m_FuncVerbUnitNo].m_bReflexive) return false;

	const CSynUnit& SitUnit = Variant.m_SynUnits[Occur.m_SituationUnitNo];
	int SitWordNo = SitUnit.m_SentPeriod.m_iFirstWord;
	const CSynHomonym&  SitHom = GetHom(Clause, Variant, Occur.m_SituationUnitNo);
	const CSynHomonym&  VerbHom = GetHom(Clause, Variant, Occur.m_FuncVerbUnitNo);
	if (!VerbHom.HasPos(gPA2)) return false;
	CPeriod  VerbPeriod = Variant.m_SynUnits[Occur.m_FuncVerbUnitNo].m_SentPeriod;
	int NP_SIMIL = Clause.GetOpt()->GetGroupTypebyName("NP_SIMIL");
	for (size_t i=0; i< Variant.m_vectorGroups.GetGroups().size(); i++)
	{
		const CGroup& G = Variant.m_vectorGroups.GetGroups()[i];
		if	(		( 
							(G.m_MainWordNo == SitWordNo)
						||	(		CPeriod(SitWordNo).is_inside_of(G)
								&&	(G.m_GroupType == NP_SIMIL)
							)
					)
				&&	VerbPeriod.is_inside_of(G)
			)
			return true;
	};

	return false;
};

// 1. Dienst and leisten is infinitive 
//  examples:
//  Der Dienst ist zu leisten.
//  er  hat  immer weniger Dienste zu leisten
//  Der Dienst ist, unter Zugrundelegung zu leisten.
//  Diese Dienste sollen geleistet werden.
bool CheckGrammar4(const CClause& Clause, const CMorphVariant& Variant,  const CCollocOccur& Occur)
{
	if (Variant.m_SynUnits[Occur.m_FuncVerbUnitNo].m_bReflexive) return false;

	const CSynHomonym&  VerbHom = GetHom(Clause, Variant, Occur.m_FuncVerbUnitNo);

	if (!IsSubjectOrSimilar(Clause, Variant,Occur.m_SituationUnitNo))
		return false;

	const CSynHomonym&  PredkHom = GetHom(Clause, Variant, Variant.m_iPredk);

	if (		VerbHom.HasPos(gZU_INFINITIV)
			&&	(		PredkHom.IsLemma("SEIN") 
					||	PredkHom.IsLemma("HABEN")	
				)
		)
		return true;
	else
		if	(		VerbHom.m_bPassive
				&&	PredkHom.HasGrammem(gModal)
			)
			return true;
		else
			return false;


};


// "leisten" is a zu-infinitive
// "Dienst" is an accusativ group 
// examples: 
// Ich gehe um einen aktiven Dienst zu leisten.
bool CheckGrammar5(const CClause& Clause, const CMorphVariant& Variant,  const CCollocOccur& Occur)
{
	if (Variant.m_SynUnits[Occur.m_FuncVerbUnitNo].m_bReflexive) return false;

	if (Variant.m_SynUnits[Occur.m_FuncVerbUnitNo].m_Type == EClause) return false;
	const CSynUnit& SitUnit = Variant.m_SynUnits[Occur.m_SituationUnitNo];

	const CSynHomonym&  VerbHom = GetHom(Clause, Variant, Occur.m_FuncVerbUnitNo);
	if (!VerbHom.HasPos(gZU_INFINITIV)) return false;

	if ( !SitUnit.HasGrammem(gAkkusativ) )
		return false;

	if (Variant.m_Subjects.size() == 1)
		if	(Variant.GetFirstSubject() == Occur.m_SituationUnitNo)
			return false;

	return true;
};


void FindInSynVariant(const CClause& Clause, const CMorphVariant& Variant,  set<string>& Result )
{
	const CSentence& Sentence = *Clause.m_pSent;
	vector<CCollocOccur >  OperHyps;
	for (int UnitNo1 = 0; UnitNo1 < Variant.m_SynUnits.size(); UnitNo1++)
	{
		if (Variant.m_SynUnits[UnitNo1].m_Type == EClause) continue;

		const CSynUnit& SitUnit  = Variant.m_SynUnits[UnitNo1];
		const CSynHomonym&  Hom1 = GetHom(Clause, Variant, UnitNo1);

		for (size_t i=0; i < OperInfosCount; i++)
		{
			const COperInfo&  Info = OperInfos[i];
			bool bSitWord =	Hom1.IsLemma(Info.m_Situation);
			if (!bSitWord) 
				if	(		(Clause.m_AntecedentWordNo != -1)
					 	&&	(Clause.m_RelativeWord.m_WordNo == SitUnit.m_SentPeriod.m_iFirstWord)
						&&	(Clause.m_RelativeWord.m_HomonymNo == SitUnit.m_iHomonymNum)
					)
				{
					if (Clause.GetWords()[Clause.m_AntecedentWordNo].FindLemma(Info.m_Situation))
						bSitWord = true;
					else
					{
						// checking the similar phrases
						int ClauseNo = Sentence.GetMinClauseByWordNo(Clause.m_AntecedentWordNo);
						assert (	ClauseNo != -1);
						int SIMIL_NP = Sentence.GetOpt()->GetGroupTypebyName("NP_SIMIL");
						assert (SIMIL_NP != -1);
						CSVI SynVar;
						const CGroup* G = Sentence.m_Clauses[ClauseNo].IsInThisGroup(Clause.m_AntecedentWordNo, SIMIL_NP, SynVar);
						if (G != 0)
							for (size_t u=G->m_iFirstWord; u <= G->m_iLastWord; u++)
							{
								int g  = SynVar->m_vectorGroups.get_maximal_subgroup(*G, u);
								if (g != -1)
								{
									const CGroup& Child = SynVar->m_vectorGroups.GetGroups()[g];
									if (Clause.GetWords()[Child.m_MainWordNo].FindLemma(Info.m_Situation))
									{
										bSitWord = true;
										break;
									};
									u = Child.m_iLastWord;
								};
						};


					};
				};

			

			if (bSitWord )
			{
				for (int UnitNo2 = 0; UnitNo2 < Variant.m_SynUnits.size(); UnitNo2++)
				{
					const CSynHomonym&  Hom2 = GetHom(Clause, Variant, UnitNo2);

					if (Hom2.IsLemma(OperInfos[i].m_FuncVerb))
					{
						CCollocOccur O;
						O.m_SituationUnitNo = UnitNo1;
						O.m_FuncVerbUnitNo = UnitNo2;
						OperHyps.push_back(O);
					};


				};

			};
		}
	}

	for (size_t i=0;  i< OperHyps.size(); i++)
	{
		const CCollocOccur& C = OperHyps[i];
		string CheckName;
		if (CheckGrammar1(Clause, Variant,  C))
		{
			CheckName = "Active, Finite";
		}
		else
		if (CheckGrammar2(Clause, Variant,  C))
		{
			CheckName = "Passive, Finite";
		}
		else
		if (CheckGrammar3(Clause, Variant,  C))
		{
			CheckName = "Nominalized";
		}
		else
		if (CheckGrammar4(Clause, Variant,  C))
		{
			CheckName = "With modal infinitive";
		}
		else
		if (CheckGrammar5(Clause, Variant,  C))
		{
			CheckName = "With infinitive";
		}
		
		
			

		if (!CheckName.empty())
		{
			string s = Format("\t%s: Sit=%s(%i), FuncVerb=%s(%i)",  
				CheckName.c_str(),
				GetHom(Clause, Variant, C.m_SituationUnitNo).m_strLemma.c_str(),
				Variant.m_SynUnits[C.m_SituationUnitNo].m_SentPeriod.m_iFirstWord,
				GetHom(Clause, Variant, C.m_FuncVerbUnitNo).m_strLemma.c_str(),
				Variant.m_SynUnits[C.m_FuncVerbUnitNo].m_SentPeriod.m_iFirstWord);

			Result.insert(s);
		};
	};

};

void FindInSentence(const CSentence& Sentence, FILE* res_fp )
{
	WriteCurrentSentence(Sentence, res_fp);

	const CAgramtab* pAgramtab = Sentence.GetOpt()->GetGramTab();
	int iWord=0,iClau=0,iCvar=0;
	
	int nClausesCount = Sentence.m_vectorPrClauseNo.size();
	bool bFound = false;
	for( int ClauseNo = 0; ClauseNo<nClausesCount; ClauseNo++ )
	{
		int PrClauseNo = Sentence.m_vectorPrClauseNo[ClauseNo];
		const CClause& Clause = Sentence.GetClause(PrClauseNo);
		int SynVarsCount = Clause.m_SynVariants.size();
		if (SynVarsCount == 0)
			throw CExpc( "No syntax variants for the current sentence");

		const CMorphVariant& V = *Clause.m_SynVariants.begin();
		int MaxWeight = V.m_iWeight;
		
		int ClauseType = (V.m_ClauseTypeNo == -1) ? UnknownSyntaxElement  :  Clause.m_vectorTypes[V.m_ClauseTypeNo].m_Type;

		set<string> Result;
		for (CSVI pSynVar = Clause.m_SynVariants.begin(); pSynVar != Clause.m_SynVariants.end(); pSynVar++)
		{
			if (pSynVar->m_iWeight < MaxWeight) break;
			FindInSynVariant(Clause, *pSynVar, Result );
		};

		for (set<string>::const_iterator it = Result.begin(); it != Result.end(); it++)
		{
			fprintf (res_fp, "%s\n", it->c_str());
			bFound = true;
		};


	};
	if (!bFound)
	{
		fprintf (res_fp, "\tNo_FVG\n");
	};
	

};


bool CheckSentence(string SentStr, CSyntaxHolder& SyntaxHolder, FILE* res_fp)
{
	if	(!SyntaxHolder.GetSentencesFromSynAn(SentStr, false))
	{
		fprintf (stderr, "Cannot load sentence %s!\n",SentStr.c_str());
		return false;
	};

	if	(SyntaxHolder.m_Synan.m_vectorSents.empty())
	{
		fprintf (stderr, "No sentence found in %s!\n",SentStr.c_str());
		return false;
	};
	try 
	{
		FindInSentence( *SyntaxHolder.m_Synan.m_vectorSents[0], res_fp );
	} 
	catch(CExpc C)
	{
		fprintf (stderr, "an exception occurred while  getting syntax information from %s : %s!\n", SentStr.c_str(), C.m_strCause.c_str() );
		return false;
	}
	catch(...)
	{
		fprintf (stderr, "an exception occurred while  getting syntax information from %s!\n", SentStr.c_str());
		return false;
	}

	return true;
};


void PrintUsage()
{
		printf ("SynColloc Collocation finder\n");
		printf ("Usage: SynColloc (RUSSIAN|GERMAN) [input-file]\n");
		printf ("Example: SynColloc German dienst100.txt\n");
		exit(-1);	
};

int main(int argc, char* argv[])
{
try {
 	
	if (argc != 3) PrintUsage();
	MorphLanguageEnum langua;
	
	
	if (!GetLanguageByString(argv[1], langua)) PrintUsage();
	CSyntaxHolder SyntaxHolder;
	if (!SyntaxHolder.LoadSyntax(langua))
	{
		fprintf (stderr, "initialization error\n");
		return 1;
	};
	fprintf (stderr, "ok\n");   	
	string FileName = argv[2];
	FILE * fp =  fopen( FileName.c_str(), "r");
	if (!fp)
	{
		printf ("Cannot open %s\n",FileName.c_str());
		return -1;
	};

	StringVector Sentences;
	char buffer[10000];
	while (fgets(buffer, 10000, fp))
	{
		string q  = buffer;
		Trim(q);
		if (q.empty()) continue;
		if (q.substr(0, 5) ==  "<lbl>") continue;
		Sentences.push_back(buffer);
	};
	fclose(fp);


	string OutFileName = MakeFName(FileName,"colloc");
	FILE* res_fp =  fopen (OutFileName.c_str(), "wb");
	if (!res_fp)
	{
		printf ("cannot open file %s!\n",OutFileName.c_str());
		return false;
	};

	//ErrorMessage("ok");
	for (size_t i=0; i < Sentences.size(); i++)
	{
		printf ("Sentence: %i/%i   \r",i+1, Sentences.size());
		if (!CheckSentence(Sentences[i], SyntaxHolder, res_fp))
		{
			fclose (res_fp);
			return 1;
		};
	};
	printf ("Sentence: %i/%i   \n",Sentences.size(), Sentences.size());
	fclose (res_fp);


}
catch(...)
{
 	fprintf (stderr, "an exception occurred!\n");   	
	return 1;
};

	return 0;
};

